﻿


using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Interactivity;
using System.Windows.Media;
using Infragistics.Windows.DataPresenter;



namespace XamDataGridDropDragTest
{


    /// <summary>
    /// CopyValidationDelegate
    /// </summary>
    /// <param name="sourceCell"></param>
    /// <param name="destinationCell"></param>
    /// <returns></returns>
    public delegate bool CopyValidationDelegate(Object sourceCell);

    /// <summary>
    /// DropValidationDelegate
    /// </summary>
    /// <param name="sourceCell"></param>
    /// <param name="destinationCell"></param>
    /// <returns></returns>
    public delegate bool DropValidationDelegate(Object sourceCell, Object destinationCell);

    /// <summary>
    /// DebuggingDelegate
    /// </summary>
    /// <param name="Text"></param>
    public delegate void DebuggingDelegate(String message);

    /// <summary>
    /// CellCopiedDelegate
    /// </summary>
    /// <param name="sourceCell"></param>
    /// <param name="destinationCell"></param>
    public delegate void CellCopiedDelegate(Object sourceCell, Object destinationCell);

    /// <summary>
    /// CellDeletedDelegate
    /// </summary>
    /// <param name="sourceCell"></param>
    public delegate void CellDeletedDelegate(Object sourceCell);


    /// <summary>
    /// ElementCopiedEventArgs
    /// </summary>
    public class ElementCopiedEventArgs : EventArgs
    {
        public Object SourceCell { get; set; }
        public Object DestinationCell { get; set; }
    }


    #region XamDataGridDragAndDropBehavior
    public class XamDataGridDragAndDropBehavior : Behavior<XamDataGrid>
    {
        


        private Cell _activeCell
        {
            get { return AssociatedObject.ActiveCell; }
        }


        private List<String> _allowedToCopyList;


        private List<String> _allowedToPasteList;


        private Cell _cellToPaste;


        private Cell _cellToCopy;


        private DataRecordPresenter _dataRecordPresenter;


        private Boolean _isButtonDown;


        private Boolean _isEscapeHit;


        private int _mouseMovementCount;


        private Popup _popUp = new Popup();


        private Label _popUpContentLabel;


        private Size _popUpSize;


        private Double _width = 250;

        private StackTrace _stackTrace;


        private Boolean _debuggingActivated;




        public event EventHandler<ElementCopiedEventArgs> ElementCopied;

        #region DependencyProperties

        public static readonly DependencyProperty AcceptedColumnsToCopyProperty =
            DependencyProperty.Register("AcceptedColumnsToCopy", typeof(String),
            typeof(XamDataGridDragAndDropBehavior), new PropertyMetadata(String.Empty));

        /// <summary>
        /// AcceptedColumnsToPasteProperty (comma-seperated)
        /// </summary>
        public static readonly DependencyProperty AcceptedColumnsToPasteProperty =
            DependencyProperty.Register("AcceptedColumnsToPaste", typeof(String),
            typeof(XamDataGridDragAndDropBehavior), new PropertyMetadata(String.Empty));

        /// <summary>
        /// CopyValidationProperty
        /// </summary>
        public static readonly DependencyProperty CopyValidationDelegateProperty =
            DependencyProperty.Register("CopyValidationDelegate", typeof(CopyValidationDelegate),
            typeof(XamDataGridDragAndDropBehavior));

        /// <summary>
        /// DropValidationProperty
        /// </summary>
        public static readonly DependencyProperty DropValidationDelegateProperty =
            DependencyProperty.Register("DropValidationDelegate", typeof(DropValidationDelegate),
            typeof(XamDataGridDragAndDropBehavior));

        /// <summary>
        /// ControllingProperty
        /// </summary>
        public static readonly DependencyProperty DebuggingDelegateProperty =
            DependencyProperty.Register("DebuggingDelegate", typeof(DebuggingDelegate),
            typeof(XamDataGridDragAndDropBehavior));

        /// <summary>
        /// CellCopiedDelegateProperty
        /// </summary>
        public static readonly DependencyProperty CellCopiedDelegateProperty =
            DependencyProperty.Register("CellCopiedDelegate", typeof(CellCopiedDelegate),
            typeof(XamDataGridDragAndDropBehavior));

        /// <summary>
        /// CellDeletedDelegateProperty
        /// </summary>
        public static readonly DependencyProperty CellDeletedDelegateProperty =
            DependencyProperty.Register("CellDeletedDelegate", typeof(CellDeletedDelegate),
            typeof(XamDataGridDragAndDropBehavior));

        /// <summary>
        /// MovementsBeforeDragBegins
        /// </summary>
        public static readonly DependencyProperty MovementsBeforeDragBeginsProperty =
            DependencyProperty.Register("MovementsBeforeDragBegins", typeof(Int32),
            typeof(XamDataGridDragAndDropBehavior), new PropertyMetadata(3));

        /// <summary>
        /// PopupBackgroundProperty
        /// </summary>
        public static readonly DependencyProperty PopupBackgroundProperty =
            DependencyProperty.Register("PopupBackground", typeof(Brush),
            typeof(XamDataGridDragAndDropBehavior), new PropertyMetadata(Brushes.White));

        /// <summary>
        /// PopupBorderBrushProperty
        /// </summary>
        public static readonly DependencyProperty PopupBorderBrushProperty =
            DependencyProperty.Register("PopupBorderBrush", typeof(Brush),
            typeof(XamDataGridDragAndDropBehavior), new PropertyMetadata(Brushes.RoyalBlue));

        /// <summary>
        /// PopupBorderBrushThicknessProperty
        /// </summary>
        public static readonly DependencyProperty PopupBorderBrushThicknessProperty =
            DependencyProperty.Register("PopupBorderThickness", typeof(Double),
            typeof(XamDataGridDragAndDropBehavior), new PropertyMetadata(1.0));

        /// <summary>
        /// PopupOpacityProperty
        /// </summary>
        public static readonly DependencyProperty PopupOpacityProperty =
            DependencyProperty.Register("PopupOpacity", typeof(Double),
            typeof(XamDataGridDragAndDropBehavior), new PropertyMetadata(1.0));
        #endregion DependencyProperties

        #region Wrapper
        /// <summary>
        /// AcceptedColumnsToCopy
        /// </summary>
        public String AcceptedColumnsToCopy
        {
            get { return (String)GetValue(AcceptedColumnsToCopyProperty); }
            set { SetValue(AcceptedColumnsToCopyProperty, value); }
        }

        /// <summary>
        /// AcceptedColumnsToPaste
        /// </summary>
        public String AcceptedColumnsToPaste
        {
            get { return (String)GetValue(AcceptedColumnsToPasteProperty); }
            set { SetValue(AcceptedColumnsToPasteProperty, value); }
        }

        /// <summary>
        /// CopyValidation
        /// </summary>
        public CopyValidationDelegate CopyValidationDelegate
        {
            get { return (CopyValidationDelegate)GetValue(CopyValidationDelegateProperty); }
            set { SetValue(CopyValidationDelegateProperty, value); }
        }

        /// <summary>
        /// DropValidation
        /// </summary>
        public DropValidationDelegate DropValidationDelegate
        {
            get { return (DropValidationDelegate)GetValue(DropValidationDelegateProperty); }
            set { SetValue(DropValidationDelegateProperty, value); }
        }

        /// <summary>
        /// DebuggingDelegate
        /// </summary>
        public DebuggingDelegate DebuggingDelegate
        {
            get { return (DebuggingDelegate)GetValue(DebuggingDelegateProperty); }
            set { SetValue(DebuggingDelegateProperty, value); }
        }

        /// <summary>
        /// CellCopiedDelegate
        /// </summary>
        public CellCopiedDelegate CellCopiedDelegate
        {
            get { return (CellCopiedDelegate)GetValue(CellCopiedDelegateProperty); }
            set { SetValue(CellCopiedDelegateProperty, value); }
        }

        /// <summary>
        /// CellDeletedDelegate
        /// </summary>
        public CellDeletedDelegate CellDeletedDelegate
        {
            get { return (CellDeletedDelegate)GetValue(CellDeletedDelegateProperty); }
            set { SetValue(CellDeletedDelegateProperty, value); }
        }

        /// <summary>
        /// MovementsBeforeDragBegins
        /// </summary>
        public Int32 MovementsBeforeDragBegins
        {
            get { return (Int32)GetValue(MovementsBeforeDragBeginsProperty); }
            set { SetValue(MovementsBeforeDragBeginsProperty, value); }
        }

        /// <summary>
        /// PopupBackground
        /// </summary>
        public Brush PopupBackground
        {
            get { return (Brush)GetValue(PopupBackgroundProperty); }
            set { SetValue(PopupBackgroundProperty, value); }
        }

        /// <summary>
        /// PopupBackground
        /// </summary>
        public Brush PopupBorderBrush
        {
            get { return (Brush)GetValue(PopupBorderBrushProperty); }
            set { SetValue(PopupBorderBrushProperty, value); }
        }

        /// <summary>
        /// PopupBorderThickness
        /// </summary>
        public Double PopupBorderThickness
        {
            get { return (Double)GetValue(PopupBorderBrushThicknessProperty); }
            set { SetValue(PopupBorderBrushThicknessProperty, value); }
        }

        /// <summary>
        /// PopupOpacity
        /// </summary>
        public Double PopupOpacity
        {
            get { return (Double)GetValue(PopupOpacityProperty); }
            set { SetValue(PopupOpacityProperty, value); }
        }
        #endregion Wrapper


        #region Methods
        /// <summary>
        /// OnAttached
        /// </summary>
        protected override void OnAttached()
        {

            if (String.IsNullOrEmpty(AcceptedColumnsToCopy))
                throw new ArgumentException("AcceptedColumnsForCopy");

            if (String.IsNullOrEmpty(AcceptedColumnsToPaste))
                throw new ArgumentException("AcceptedColumnsForPaste");

            _allowedToCopyList = new List<String>();
            foreach (String item in AcceptedColumnsToCopy.Split(','))
            {
                String withoutSpaces = item.Trim();
                _allowedToCopyList.Add(withoutSpaces);
            }

            _allowedToPasteList = new List<String>();
            foreach (String item in AcceptedColumnsToPaste.Split(','))
            {
                String withoutSpaces = item.Trim();
                _allowedToPasteList.Add(withoutSpaces);
            }

            AssociatedObject.PreviewMouseLeftButtonDown += new MouseButtonEventHandler(_xamDataGrid_PreviewMouseLeftButtonDown);
            AssociatedObject.PreviewMouseLeftButtonUp += new MouseButtonEventHandler(_xamDataGrid_PreviewMouseLeftButtonUp);
            AssociatedObject.PreviewKeyDown += new KeyEventHandler(_xamDataGrid_PreviewKeyDown);
            AssociatedObject.PreviewMouseMove += new MouseEventHandler(_xamDataGrid_PreviewMouseMove);
        }

        #region Eventhandling
        /// <summary>
        /// _xamDataGrid_PreviewKeyDown
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void _xamDataGrid_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Escape)
            {
                if (_debuggingActivated)
                    //SendDebuggingMessage(_stackTrace.GetFrame(0).GetMethod().Name, _stackTrace.GetFrame(1).GetMethod().Name, e.Key.ToString());

                    _isEscapeHit = true;
                CleanUpView();
            }
            if (e.Key == Key.Delete)
            {
                ItemDeleted();
            }


        }

        /// <summary>
        /// _xamDataGrid_PreviewMouseMove
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void _xamDataGrid_PreviewMouseMove(object sender, MouseEventArgs e)
        {
            if (DebuggingDelegate != null && _stackTrace == null)
            {
                _debuggingActivated = true;
                _stackTrace = new StackTrace();
            }
            if (_mouseMovementCount <= MovementsBeforeDragBegins)
            {
                _mouseMovementCount++;
                return;
            }


            if (_debuggingActivated && (count == 25 || count == 1))
            {
                //SendDebuggingMessage(_stackTrace.GetFrame(0).GetMethod().Name, _stackTrace.GetFrame(1).GetMethod().Name, "Moved " + count.ToString(CultureInfo.InvariantCulture));
                if (count == 1)
                {
                    count++;
                }
                if (count == 25)
                    count = 1;
            }
            else
            {
                count++;
            }

            if (_isButtonDown)
            {
                if (_cellToCopy == null)
                {
                    _cellToCopy = _activeCell;
                }
                if (ValidateCopy())
                {
                    UpdatePopup(e);
                }
            }
        }
        private int count;
        /// <summary>
        /// _xamDataGrid_PreviewMouseLeftButtonUp
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void _xamDataGrid_PreviewMouseLeftButtonUp(Object sender, MouseButtonEventArgs e)
        {
            CleanUpView();
            _cellToPaste = _activeCell;

            this._isButtonDown = false;
            if (ValidateCopy() && ValidateDrop())
            {
                Object Record = _cellToPaste.Record.DataItem;
                Object DataToChange = _cellToCopy.Value;
                Record.GetType().GetProperty(_cellToPaste.Field.Name).SetValue(Record, DataToChange, null);
                OnElementCopied();
            }
            _cellToCopy = null;
            _cellToPaste = null;
            _isEscapeHit = false;

            //if (_debuggingActivated)
            //SendDebuggingMessage(_stackTrace.GetFrame(0).GetMethod().Name, _stackTrace.GetFrame(1).GetMethod().Name, "");

        }

        /// <summary>
        /// _xamDataGrid_PreviewMouseLeftButtonDown
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void _xamDataGrid_PreviewMouseLeftButtonDown(Object sender, MouseButtonEventArgs e)
        {
            _mouseMovementCount = 0;
            _isButtonDown = true;
            CleanUpView();
            DependencyObject source = e.OriginalSource as DependencyObject;
            _dataRecordPresenter = Infragistics.Windows.Utilities.GetAncestorFromType(source, typeof(DataRecordPresenter), true) as DataRecordPresenter;

            if (_debuggingActivated)
            {
                SendDebuggingMessage(_stackTrace.GetFrame(0).GetMethod().Name, _stackTrace.GetFrame(1).GetMethod().Name, "Position :" + e.GetPosition(null).ToString());
            }
        }
        #endregion Eventhandling

        #region OtherMethods
        /// <summary>
        /// CleanUp
        /// </summary>
        private void CleanUpView()
        {
            _popUp.IsOpen = false;
            _popUpContentLabel = null;
        }

        /// <summary>
        /// Creates the Popup while Dragging
        /// </summary>
        /// <param name="lbl"></param>
        private void CreatePopup(Label lbl)
        {

            if (_cellToCopy.Field.Width != null)
            {
                _width = _cellToCopy.Field.Width.Value.Value;
            }
            _popUpSize = new Size(_width, _dataRecordPresenter.ActualHeight);
            _popUp.Child = lbl;
            _popUp.AllowsTransparency = true;
            _popUp.Focusable = false;
            _popUp.IsOpen = true;
            _popUp.Placement = PlacementMode.AbsolutePoint;
        }


        /// <summary>
        /// UpdatePopup
        /// </summary>
        private void UpdatePopup(MouseEventArgs e)
        {
            if (!_isButtonDown || _isEscapeHit)
            {
                CleanUpView();
                return;
            }

            if (_isButtonDown)
            {
                if (_popUpContentLabel == null)
                {
                    _popUpContentLabel = new Label();
                    _popUpContentLabel.Content = _cellToCopy.Value;
                    _popUpContentLabel.BorderThickness = new Thickness(PopupBorderThickness);
                }

                _popUpContentLabel.Background = PopupBackground;
                _popUpContentLabel.Opacity = PopupOpacity;
                _popUpContentLabel.BorderBrush = Brushes.Transparent;
                if (!ValidateDrop())
                {
                    _popUpContentLabel.BorderBrush = PopupBorderBrush;
                }
            }

            if (_popUpContentLabel != null)
            {
                CreatePopup(_popUpContentLabel);
            }





            ////////////
            ////////////
            ////////////
            ////////////
            ////////////
            ////////////
            ////////////
            ////////////
            //////////// PROBLEM
            _popUp.PlacementRectangle = new Rect(_popUpSize);
            _popUp.HorizontalOffset = e.GetPosition(null).X;
            _popUp.VerticalOffset = e.GetPosition(null).Y;
            ////////////
            ////////////
            ////////////
            ////////////
            ////////////
            ////////////
            ////////////
            ////////////
            ////////////

        }

        /// <summary>
        /// ValidateDrop
        /// </summary>
        /// <returns></returns>
        private bool ValidateDrop()
        {

            if (_activeCell != null && _cellToCopy != null && !_isEscapeHit)
            {
                Boolean retValue;

                retValue = _activeCell.Field.DataType == _cellToCopy.Field.DataType &&
                            (_allowedToPasteList.Contains(_activeCell.Field.Name)) &&
                            _cellToCopy != _activeCell;

                if (retValue && DropValidationDelegate != null)
                    return DropValidationDelegate.Invoke(_cellToCopy.Value, _activeCell.Value);

                return retValue;
            }
            return false;
        }

        /// <summary>
        /// ValidateCopy
        /// </summary>
        /// <returns></returns>
        private bool ValidateCopy()
        {
            if (_cellToCopy != null && _allowedToCopyList.Contains(_cellToCopy.Field.Name))
            {
                if (CopyValidationDelegate != null)
                {
                    return CopyValidationDelegate.Invoke(_cellToCopy.Value);
                }
                else
                {
                    return true;
                }
            }
            return false;
        }


        /// <summary>
        /// SendDebuggingMessage
        /// </summary>
        /// <param name="method"></param>
        /// <param name="callingMethod"></param>
        /// <param name="message"></param>
        private void SendDebuggingMessage(String method, String callingMethod, String message)
        {
            DebuggingDelegate.Invoke("Method: " + method + " Message: " + message + " (CallingMethod: " + callingMethod + ", Time: " + DateTime.Now.ToString() + ")");
        }

        /// <summary>
        /// ItemDeleted
        /// </summary>
        private void ItemDeleted()
        {
            if (_activeCell != null && CellDeletedDelegate != null)
            {
                CellDeletedDelegate.Invoke(_activeCell.Value);
            }
        }
        #endregion OtherMethods
        #endregion Methods

        #region Events
        /// <summary>
        /// OnElementCopied
        /// </summary>
        protected void OnElementCopied()
        {
            if (ElementCopied != null)
            {
                ElementCopiedEventArgs args = new ElementCopiedEventArgs();
                args.SourceCell = _cellToCopy.Value;
                args.DestinationCell = _cellToPaste.Value;
                ElementCopied(this, args);
            }
            if (CellCopiedDelegate != null)
            {

                CellCopiedDelegate.Invoke(_cellToCopy.Value, _cellToPaste.Value);
            }
        }
        #endregion Events




    }

    #endregion





}



